/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        user.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/******************************************************************************
 * CDC RS-232 Emulation Tutorial Instructions:
 ******************************************************************************
 * Refer to Application Note AN956 for explanation of the CDC class.
 * 
 * First take a look at Exercise_Example() and study how functions are called.
 * 
 * There are five exercises, each one has a solution in the CDC\user\solutions.
 * Scroll down and look for Exercise_01,_02,_03,_04, and _05.
 * Instructions on what to do is inside each function.
 * 
 *****************************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"
#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"
 

/** V A R I A B L E S ********************************************************/
#pragma udata


char input_buffer[64];
char my_buffer[64];
char output_buffer[32];
char sample;
unsigned int counter,mPtr,index,inPtr,outPtr,Ost,Ist,Ast;
volatile unsigned char DATA,ch,OutputState,Ach;
 

#define SCLK PORTCbits.RC1      
#define SDATA  PORTCbits.RC0
#define CS	 PORTCbits.RC6
#define RES    PORTCbits.RC7
#define POWER PORTBbits.RB2

#define D_C PORTCbits.RC2
#define SW PORTBbits.RB0         /* unused */
 
 






 
/** P R I V A T E  P R O T O T Y P E S ***************************************/
void InitializeUSART(void);
 
 
void User_Process(void);
 void Check_Update_Input(void);
void update_bit(unsigned char Chan,unsigned char Data);
void GetInputStatus(void);
 void writecom(char command_in);                                                  
void clearram(void);                                                                         
void initlcd(void);                                                                              
void delay10( char n);                                                                       
void clockdata(char bits_in);                                                             
void cursorxy(char x, char y);                                                          
void writedata(char data_in);                                                            
void lcdchar (char c );                                                                       
void EEPROM_Write(unsigned int address,  unsigned char data);   
void EEPROM_Read(unsigned int address);                                    
void readeeprom();                                                                            
#pragma romdata ROOT_TABLE=0x1000
 
 
 
 
 
rom const char table []  [5]= {0x00,0x00,0x00,0x00,0x00,	// 20 space		 
	                   0x00,0x00,0x5f,0x00,0x00, // 21    
				0x00,0x07,0x00,0x07,0x00,	// 22 "			
	0x14,0x7f,0x14,0x7f,0x14,	// 23 #			 

							0x24,0x2a,0x7f,0x2a,0x12,	// 24 $	
			0x23,0x13,0x08,0x64,0x62,	// 25 %
	0x36,0x49,0x55,0x22,0x50,	// 26 &
	0x00,0x05,0x03,0x00,0x00,	// 27 '
	0x00,0x1c,0x22,0x41,0x00,	// 28 (
	0x00,0x41,0x22,0x1c,0x00,	// 29 )
	0x14,0x08,0x3e,0x08,0x14,	// 2a *
	0x08,0x08,0x3e,0x08,0x08,	// 2b +
	0x00,0x50,0x30,0x00,0x00,	// 2c ,
	0x08,0x08,0x08,0x08,0x08,	// 2d -
	0x00,0x60,0x60,0x00,0x00,	// 2e .
	0x20,0x10,0x08,0x04,0x02,	// 2f /
	0x3e,0x51,0x49,0x45,0x3e,	// 30 0
	0x00,0x42,0x7f,0x40,0x00,	// 31 1
	0x42,0x61,0x51,0x49,0x46,	// 32 2
	0x21,0x41,0x45,0x4b,0x31,	// 33 3
	0x18,0x14,0x12,0x7f,0x10,	// 34 4
	0x27,0x45,0x45,0x45,0x39,	// 35 5
	0x3c,0x4a,0x49,0x49,0x30,	// 36 6
	0x01,0x71,0x09,0x05,0x03,	// 37 7
	0x36,0x49,0x49,0x49,0x36,	// 38 8
	0x06,0x49,0x49,0x29,0x1e,	// 39 9
	0x00,0x36,0x36,0x00,0x00,	// 3a :
	0x00,0x56,0x36,0x00,0x00,	// 3b ;
	0x08,0x14,0x22,0x41,0x00,	// 3c <
	0x14,0x14,0x14,0x14,0x14,	// 3d =
	0x00,0x41,0x22,0x14,0x08,	// 3e >
	0x02,0x01,0x51,0x09,0x06,	// 3f ?
	0x32,0x49,0x79,0x41,0x3e,	// 40 @
	0x7e,0x11,0x11,0x11,0x7e,	// 41 A
	0x7f,0x49,0x49,0x49,0x36,	// 42 B
	0x3e,0x41,0x41,0x41,0x22,	// 43 C
	0x7f,0x41,0x41,0x22,0x1c,	// 44 D
	0x7f,0x49,0x49,0x49,0x41,	// 45 E
	0x7f,0x09,0x09,0x09,0x01,	// 46 F
	0x3e,0x41,0x49,0x49,0x7a,	// 47 G
	0x7f,0x08,0x08,0x08,0x7f,	// 48 H
	0x00,0x41,0x7f,0x41,0x00,	// 49 I
	0x20,0x40,0x41,0x3f,0x01,	// 4a J
	0x7f,0x08,0x14,0x22,0x41,	// 4b K
	0x7f,0x40,0x40,0x40,0x40,	// 4c L
	0x7f,0x02,0x0c,0x02,0x7f,	// 4d M
	0x7f,0x04,0x08,0x10,0x7f,	// 4e N
	0x3e,0x41,0x41,0x41,0x3e,	// 4f O
	0x7f,0x09,0x09,0x09,0x06,	// 50 P
	0x3e,0x41,0x51,0x21,0x5e,	// 51 Q
	0x7f,0x09,0x19,0x29,0x46,	// 52 R
	0x46,0x49,0x49,0x49,0x31,	// 53 S
	0x01,0x01,0x7f,0x01,0x01,	// 54 T
     0x3f, 0x40,0x40,0x40,0x3f,         // 55 U
	0x1f,0x20,0x40,0x20,0x1f,	// 56 V
	0x3f,0x40,0x38,0x40,0x3f,	// 57 W
	0x63,0x14,0x08,0x14,0x63,	// 58 X
	0x07,0x08,0x70,0x08,0x07,	// 59 Y
	0x61,0x51,0x49,0x45,0x43,	// 5a Z
	0x00,0x7f,0x41,0x41,0x00,	// 5b [
	0x02,0x04,0x08,0x10,0x20,	// 5c Yen Currency  
	0x00,0x41,0x41,0x7f,0x00,	// 5d ]
	0x04,0x02,0x01,0x02,0x04,	// 5e ^
	0x40,0x40,0x40,0x40,0x40,	// 5f _
	0x00,0x01,0x02,0x04,0x00,	// 60 `
	0x20,0x54,0x54,0x54,0x78,	// 61 a
	0x7f,0x48,0x44,0x44,0x38,	// 62 b
	0x38,0x44,0x44,0x44,0x20,	// 63 c
	0x38,0x44,0x44,0x48,0x7f,	// 64 d
	0x38,0x54,0x54,0x54,0x18,	// 65 e
	0x08,0x7e,0x09,0x01,0x02,	// 66 f
	0x0c,0x52,0x52,0x52,0x3e,	// 67 g
	0x7f,0x08,0x04,0x04,0x78,	// 68 h
	0x00,0x44,0x7d,0x40,0x00,	// 69 i
	0x20,0x40,0x44,0x3d,0x00,	// 6a j 
	0x7f,0x10,0x28,0x44,0x00,	// 6b k
	0x00,0x41,0x7f,0x40,0x00,	// 6c l
	0x7c,0x04,0x18,0x04,0x78,	// 6d m
	0x7c,0x08,0x04,0x04,0x78,	// 6e n
	0x38,0x44,0x44,0x44,0x38,	// 6f o
	0x7c,0x14,0x14,0x14,0x08,	// 70 p
	0x08,0x14,0x14,0x18,0x7c,	// 71 q
	0x7c,0x08,0x04,0x04,0x08,	// 72 r
	0x48,0x54,0x54,0x54,0x20,	// 73 s
	0x04,0x3f,0x44,0x40,0x20,	// 74 t
	0x3c,0x40,0x40,0x20,0x7c,	// 75 u
	0x1c,0x20,0x40,0x20,0x1c,	// 76 v
	0x3c,0x40,0x30,0x40,0x3c,	// 77 w
	0x44,0x28,0x10,0x28,0x44,	// 78 x
	0x0c,0x50,0x50,0x50,0x3c,	// 79 y
	0x44,0x64,0x54,0x4c,0x44,	// 7a z
	0x00,0x08,0x36,0x41,0x00,	// 7b <
	0x00,0x00,0x7f,0x00,0x00,	// 7c |
	0x00,0x41,0x36,0x08,0x00,	// 7d >
	0x10,0x08,0x08,0x10,0x08,	// 7e Right Arrow 	->
	0x78,0x46,0x41,0x46,0x78};	// 7f Left Arrow	<-


 


#pragma code
void UserInit(void)
{

   
    inPtr=outPtr=Ost=Ist=Ast=0;  
   
     initlcd();                       /* initialise the display */


   PORTD=0xff;
      readeeprom();                
}//end UserInit


 
void ProcessIO(void)
{   
   
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;

    User_Process();

}//end ProcessIO

 
     
 
	
//----------------------------
// Update Output                                    MAIN PROGRAM FOR PDA
//----------------------------
void Check_Update_Output(void)
{
   EEPROM_Write(sample,DATA);  /*save ASCII data in EEPROM */




  lcdchar(DATA);                       /*       write ASCII received from USB usart to display */



PORTD=0xff;



   writedata(0);
 
  if(DATA=='*')

{    
     clearram();
   }



  if(DATA=='1')

{    
     sample=0;
   }


  if(DATA=='2')

{    
     sample=84;
   }




  if(DATA=='3')

{    
    sample=168;
   }









}

  
//****************************************
// User change here
//****************************************
void User_Process(void)
{
	

    if(getsUSBUSART(input_buffer,32))
    {
		mPtr = mCDCGetRxLength();
		for(index = 0; index < mPtr; index++)
		{  		
		   my_buffer[inPtr]= input_buffer[index];
		   inPtr++;
		   if (inPtr==64) inPtr=0;
		}  	    
    }
        
    if (inPtr!=outPtr)
    {
	      DATA=my_buffer[outPtr];
	      outPtr++;
	      if (outPtr==64) outPtr=0;

		  Check_Update_Output();
    	      
    	  
	}      
	    
}//end 

 











 

void initlcd(void)     
{      sample=0;
      POWER =1;      

       SleepMS(10);	// Wait 100ms. 

	 RES = 1;		// Set _RES HIGH.
	 SCLK = 1;		// Disable Chip.
	 RES = 0;		// Reset the LCD.

   SleepMS(10);	// Wait 100ms. 
 
    RES = 1;		// Awake LCD from RESET state.

	writecom(0x21);		// Activate Chip and H=1.
	writecom(0xc8);		// c2 Set LCD Voltage to     about 7V.   vop
	writecom(0x13);		// Adjust voltage bias.
	writecom(0x20);		// Horizontal addressing and H=0.
	writecom(0x09);		// Activate all segments.
	 clearram();    	                  // Erase all pixels on the DDRAM.
	writecom(0x08);		// Blank the Display.


      SleepMS(10);	// Wait 100ms.
	writecom(0x0C);		// Display Normal.
                  writecom(0x40);
	writecom(0x80);	
    

	cursorxy(0,0);		// Cursor Home.
     SleepMS(10);	// Wait 100ms. 
      
	 clearram();
     SleepMS(10);	// Wait 100ms.
}

void clearram(void)                                         
{
	int ddram;
   sample=0;
	cursorxy(0,0);											// Cursor Home.
	for (ddram=504;ddram>0;ddram--)	{writedata(0x00);}		// 6*84 = 504 DDRAM addresses.
}



void cursorxy(char x, char y)
{
	writecom(0x40|(y&0x07));	// Y axis
	writecom(0x80|(x&0x7f));	// X axis
}




void writecom(char command_in)
{
	D_C = 0;				// Select Command register.
    CS = 0;				// Select Chip.
	clockdata(command_in);	// Clock in command bits.
	CS = 1;				// Deselect Chip.	
}

void writedata(char data_in)
{
	D_C = 1;				// Select Data register.
	CS = 0;				// Select Chip.
	clockdata(data_in);		// Clock in data bits.
	CS = 1;				// Deselect Chip.	
}









void clockdata(char bits_in)
{
	int bitcnt;
	for (bitcnt=8; bitcnt>0; bitcnt--)
		{
			SCLK = 0;								// Set Clock Idle level LOW.
			if ((bits_in&0x80)==0x80) {SDATA=1;}		// PCD8544 clocks in the MSb first.
			else {SDATA=0;}
			SCLK = 1;								// Data is clocked on the rising edge of SCK.
			bits_in=bits_in<<1;						// Logical shift data by 1 bit left.
		}
}






void InitializeUSART(void)
{
    TRISCbits.TRISC7=1; // RX
    TRISCbits.TRISC6=0; // TX
    SPBRG = 0x71;
    SPBRGH = 0x02;      // 0x0271 for 48MHz -> 19200 baud
    TXSTA = 0x24;       // TX enable BRGH=1
    RCSTA = 0x90;       // continuous RX
    BAUDCON = 0x08;     // BRG16 = 1
}//end InitializeUSART




 













 
 
void lcdchar (char c )
{
 
 int i;
 char s;
for (i=0; i<5; i++)

{
 
 writedata(table[c-32]  [i]  ); 
 }
}

 
 
void  EEPROM_Write(unsigned int address,  unsigned char data)
{


	unsigned char return_value;
                 unsigned char temp_GIEH;
	unsigned char temp_GIEL;

 
		// save the state of the interrupt enable bits
	                  temp_GIEH = INTCONbits.GIEH;
		temp_GIEL = INTCONbits.GIEL;
	
		// EEPROM operation (as opposed to a flash memory operation)
	    EECON1bits.EEPGD = 0;
	
		// Only do a write
	    EECON1bits.FREE = 0;  
	
	    // make sure the EEPROM write done flag is reset
		PIR2bits.EEIF = 0;
	
		// set EEPROM address
	    EEADR =sample ;
	    EEDATA = data;
	    EECON1bits.WREN = 1;
	
	    	INTCONbits.GIEH = 0;
		INTCONbits.GIEL = 0;
	    
	     
	    EECON2 = 0x55;
	    EECON2 = 0xAA;
	    
	 
		EECON1bits.WR = 1;

	 
	    INTCONbits.GIEH = temp_GIEH;
	    INTCONbits.GIEL = temp_GIEL;

	   
		while(PIR2bits.EEIF == 0);
	
	 
	    PIR2bits.EEIF = 0;
	
	    sample=sample+1;
	}
 
 
void EEPROM_Read(unsigned int address)
{
 
     EECON1bits.EEPGD = 0; 
     EEADR = sample;
     EECON1bits.RD = 1;
  
}

void readeeprom()       

{ char numbers;
  char next1;
  char next2;

   

 for (next2=0; next2<3;next2++)   

{

  for (next1=0; next1<6;next1++)   
{
  for (numbers=0; numbers<14; numbers++)   


{
   EEPROM_Read(sample);
  
   lcdchar(EEDATA);       
    writedata(0);              
   sample=sample+1;        
}

}

 SleepMS(9000);         
 SleepMS(9000);
 SleepMS(9000);
 SleepMS(9000);

PORTD=0 ;


  }

 
}

 


















































 
     




  